local ship = {}

ship.img = love.graphics.newImage("ship.png")

function ship:draw()
  local alpha = self:getDead() and 255*self:getDead()/death_time or 255
  if self:getTeam() == 1 then
    love.graphics.setColor(255,127,127,alpha)
  else
    love.graphics.setColor(127,255,127,alpha)
  end
  love.graphics.draw(ship.img,self:getX(),self:getY(),self:getR(),0.25,0.25,64,64)
  love.graphics.setColor(255,255,255)
  if not debug then
    if self:getDead() then
      love.graphics.setColor(255,0,0)
    end
    love.graphics.rectangle("line",
      self:getX()-self:getRange(),
      self:getY()-self:getRange(),
      self:getRange()*2,
      self:getRange()*2)
    love.graphics.circle("line",
      self:getX(),self:getY(),self:getRange())
    love.graphics.setColor(255,255,255,31)
    love.graphics.line(self:getX(),self:getY(),self:getTx(),self:getTy())
    love.graphics.setColor(255,255,255)
  end
end

function ship:update(dt)
  if self:getDead() then
    self:setDead( self:getDead() - dt )
    return 0
  end
  if self:getTx() == nil and self:getTy() == nil then
    self:setTx(math.random(0,love.graphics.getWidth()))
    self:setTy(math.random(0,love.graphics.getHeight()))
    self:setR(0)
    return 0
  else
    self:setR( math.atan2(self:getX()-self:getTx(),self:getTy()-self:getY())+math.pi/2)
    local checks_count = 0
    if use_bump then
      local target_x = self:getX()+(math.cos(self:getR())*dt*100)-self:getRange()
      local target_y = self:getY()+(math.sin(self:getR())*dt*100)-self:getRange()
      -- Check what you're going to collide with
      local cactual_x, cactual_y, ccols, clen = world:check(self,target_x,target_y,
        function(item,other)
          if item:getTeam() == other:getTeam() then return nil else return "slide" end
        end)
      -- Move that ship anyway!
      local actual_x, actual_y, cols, len = world:move(self,target_x,target_y,function() return nil end)

      self:setX(target_x+self:getRange())
      self:setY(target_y+self:getRange())
      for ci,cv in pairs(ccols) do
        if not cv.other:getDead() then
          local dist = self.distance(self:getX(),self:getY(),
            cv.other:getX(),cv.other:getY())
          checks_count = checks_count + 1
          if dist < range*2 then
            cv.other:setDead(death_time)
            self:setDead(death_time)
          end
        end
      end
    else -- no bump
      local target_x = self:getX()+(math.cos(self:getR())*dt*100)
      local target_y = self:getY()+(math.sin(self:getR())*dt*100)
      self:setX(target_x)
      self:setY(target_y)
      for otheri,otherv in pairs(ships) do
        if not otherv:getDead() and otherv ~= self and self:getTeam() ~= otherv:getTeam() then
          local dist = self.distance(self:getX(),self:getY(),
            otherv:getX(),otherv:getY())
          checks_count = checks_count + 1
          if dist < range*2 then
            otherv:setDead(death_time)
            self:setDead(death_time)
          end
        end
      end
    end
    if self.distance(self:getX(),self:getY(),
      self:getTx(),self:getTy()) < range/4 then
      self:setTx(nil)
      self:setTy(nil)
    end
    return checks_count
  end
end

function ship.distance(x1,y1,x2,y2)
  return math.sqrt((x1-x2)^2+(y1-y2)^2)
end

-- LuaClassGen pregenerated functions

function ship.new(init)
  init = init or {}
  local self={}
  self.draw=ship.draw
  self.update=ship.update
  self.distance=ship.distance
  self._x=init.x
  self.getX=ship.getX
  self.setX=ship.setX
  self._y=init.y
  self.getY=ship.getY
  self.setY=ship.setY
  self._tx=init.tx
  self.getTx=ship.getTx
  self.setTx=ship.setTx
  self._ty=init.ty
  self.getTy=ship.getTy
  self.setTy=ship.setTy
  self._r=init.r
  self.getR=ship.getR
  self.setR=ship.setR
  self._range=init.range
  self.getRange=ship.getRange
  self.setRange=ship.setRange
  self._dead=init.dead
  self.getDead=ship.getDead
  self.setDead=ship.setDead
  self._team=init.team
  self.getTeam=ship.getTeam
  self.setTeam=ship.setTeam
  return self
end

function ship:getX()
  return self._x
end

function ship:setX(val)
  self._x=val
end

function ship:getY()
  return self._y
end

function ship:setY(val)
  self._y=val
end

function ship:getTx()
  return self._tx
end

function ship:setTx(val)
  self._tx=val
end

function ship:getTy()
  return self._ty
end

function ship:setTy(val)
  self._ty=val
end

function ship:getR()
  return self._r
end

function ship:setR(val)
  self._r=val
end

function ship:getRange()
  return self._range
end

function ship:setRange(val)
  self._range=val
end

function ship:getDead()
  return self._dead
end

function ship:setDead(val)
  self._dead=val
end

function ship:getTeam()
  return self._team
end

function ship:setTeam(val)
  self._team=val
end

return ship
